import javax.swing.*;

//

float lerpAng(float ang, float dest, float damp)
{
  float diff = dest - ang;
  while (diff > PI) diff -= TWO_PI; // unwinding
  while (diff < -PI) diff += TWO_PI;
  return ang + (diff) * damp;
}

void resetComposer()
{
  actionFrame = 0;
  frame = 0;
  numFrames = 0;
  
  active = 0;
  mover = movers.get(active);
  
  resetMovers();
  
  date = "";
}

void resetMovers()
{
  for (Mover m : movers) m.reset();
}

Mover getLongestMover()
{
 int max = 0;
 
 Mover m = null;

 for (int i = 0; i < movers.size(); i++) {
   m = movers.get(i);
   if (m.positions.size() > max) max = m.positions.size();
 }
 
 return m;
}

void saveLevel()
{
  JSONObject jlevel = new JSONObject();
  
  jlevel.setFloat("length", numFrames);
  jlevel.setString("date", date);
  jlevel.setInt("fritz", active);
  jlevel.setInt("frame", actionFrame);
  jlevel.setInt("future", frame);
  
  JSONArray jmovers = new JSONArray();
  int inc = 0;
  
  for (int k = 0; k < movers.size(); k++) {
    Mover m = movers.get(k);
    
    if (m.positions.size() == 0) continue;
    
    JSONObject jmover = new JSONObject();
    jmover.setInt("type", m.type);
    jmover.setBoolean("state", m.state);
    
    JSONArray jframes = new JSONArray();
    
    for (int i = 0; i < m.positions.size(); i++) {
      PVector p = m.positions.get(i);
      
      JSONObject jframe = new JSONObject();
      jframe.setFloat("x", map(p.x, 0, width, 0, 1));
      jframe.setFloat("y", map(p.y, 0, height, 0, 1));
      jframe.setFloat("rotation", m.rotations.get(i));
      
      jframes.setJSONObject(i, jframe);    
    }
    
    jmover.setJSONArray("frames", jframes);
    jmovers.setJSONObject(inc, jmover);
    inc++;
  }
  
  jlevel.setJSONArray("movers", jmovers);
  
  saveJSONObject(jlevel, "data/NEW_LEVEL.json"); 
}

void importScene()
{
  launchChooser();
}

void launchChooser(){
 SwingUtilities.invokeLater(new Runnable() {
   public void run() {
     try {
       JFileChooser fc = new JFileChooser(sketchPath+"/data");
        int returnVal = fc.showOpenDialog(null);
        if (returnVal == JFileChooser.APPROVE_OPTION) {
          File file = fc.getSelectedFile();
          String name = file.getName().toLowerCase();
          if (name.endsWith(".json")) {
            reset = true;
            waiting = true;
            imported = name;
          }
        }
     } catch (Exception e) {
       e.printStackTrace();
     }
   }
 });
}

void importLevel(String levelName)
{
  JSONObject level = loadJSONObject(levelName);
  
  //SET
  numFrames = level.getInt("length");
  active = level.getInt("fritz");
  mover = movers.get(active);
  actionFrame = level.getInt("frame");
  frame = level.getInt("future");
 
  JSONArray jsonMovers = level.getJSONArray("movers");
  println(jsonMovers.size());
   
  for (int i = 0; i < jsonMovers.size(); i++) {
    JSONObject jsonMover = jsonMovers.getJSONObject(i);
    
    //SET
    Mover m = movers.get(i);
    m.type = jsonMover.getInt("type");
    m.state = jsonMover.getBoolean("state");
   
    JSONArray jsonFrames = jsonMover.getJSONArray("frames");
   
    for (int j = 0; j < jsonFrames.size(); j++) {
      JSONObject jsonFrame = jsonFrames.getJSONObject(j);
      float x = jsonFrame.getFloat("x");
      float y = jsonFrame.getFloat("y");
      
      //SET
      m.positions.add(new PVector(x * width, y * height));
      float rotation = jsonFrame.getFloat("rotation");
      m.rotations.add(new Float(rotation));
    }
  }
}
