import java.util.Date;
import java.text.DateFormat;
import java.text.SimpleDateFormat;

//

int mode = 2;
int frame = 0;
int numFrames = 0;

PVector ori;
PVector vec;
float dang;

ArrayList<PImage> actors;

ArrayList<Mover> movers;
Mover mover;

int active = 0;

String date = "";

float imageOffset = PI;

int actionFrame = 0;

boolean help;

boolean toggle;

boolean reset = false;
boolean waiting = false;
String imported;

void setup()
{
  size(640, 480);
  textAlign(LEFT);
  
  actors = new ArrayList();
  actors.add(loadImage("actors/triangle.png"));
  actors.add(loadImage("actors/square.png"));
  actors.add(loadImage("actors/circle.png"));
  
  movers = new ArrayList();
  for (int i = 0; i < 9; i++) {
    movers.add(new Mover());
  }
  
  mover = movers.get(0);
}

void draw()
{
  color c = (toggle) ? color(150) : color(200);
  background(c);
  
  //
  
  if (mode == 1) {
    if (mousePressed) {
      mover.positions.add(new PVector(mouseX, mouseY));
      mover.rotations.add(0.0f);
      if (mover.positions.size() > numFrames) numFrames = mover.positions.size();
      frame = mover.positions.size()-1;
    }
  } else if (mode == 1 || mode == 3) {
    frame = (frame + 1) % numFrames;
  }
  
  if (mode == 3 && ori != null) {
    stroke(0, 255, 0, 150);
    noFill();
    ellipse(ori.x, ori.y, 120, 120);
    line(ori.x, ori.y, mouseX, mouseY);
    
    if (mover.rotations.size() > 0) {
      Float newVal = new Float(dang - imageOffset);
      int index = constrain(frame, 0, mover.rotations.size()-1);
      mover.rotations.set(index, newVal);
    }
  }
  
  //
  
  for (int i = 0; i < movers.size(); i++) {
    Mover m = movers.get(i);
    m.update();
    
    if (m.positions.size() > 0) {
      int index = constrain(frame, 0, m.positions.size()-1);
      fill(0);
      text("m : " + (i+1), m.positions.get(index).x, m.positions.get(index).y - 25);
    }
  }
  
  //
  
  fill(0);
  rect(width-55, height-25, 55, 25);
  fill(0, 255, 0);
  text("m : " + (active+1), width-40, height-10);
  
  fill(0);
  rect(0, height-25, 200, 25);
  fill(0, 255, 0);
  text("saved : " + date, 10, height-10);
  
  fill(0);
  rect(width-70, 0, 70, 25);
  fill(0, 255, 0);
  text("f : " + actionFrame, width-55, 15);
  
  if (help) {
    fill(255);
    noStroke();
    rect(15, 15, 300, height-30);
    fill(0);
    text("fritz composer", 25, 35);
    
    text("- drag mouse to record a path", 25, 60);
    text("- press 1-9 to change the actor", 25, 80);
    text("- press c to change the skin of the actor", 25, 100);
    text("- toggle d to change state", 25, 120);
    text("- toggle space to choose edit/play mode", 25, 140);
    
    text("- edit mode :", 25, 170);
    text("  - left/right arrows to change the frame", 25, 190);
    text("  - press f to choose the action frame", 25, 210);
    text("  - up/down arrows to change the angle", 25, 230);
    text("  - press i to import a level", 25, 250);
    text("  - press r to reset the level", 25, 270);
    
    text("- play mode : drag mouse to record angle", 25, 300);
    text("  - drag mouse to record angle", 25, 320);
    
    text("- press s to save a json file of the level", 25, 350);
    text("- fritz will be the active actor", 25, 370);
    text("- begin will be the action frame", 25, 390);
    text("- final will be the active frame", 25, 410);
  } else {
    fill(255);
    text("help : press h", 15, 25);
  }
  
  fill(155, 0, 0);
  float rw = map(frame, 0, numFrames, 0, width);
  rect(0, 0, rw, 5);
  
  //
  
  if (reset) {
    reset = false;
    resetComposer();
    if (waiting) {
      waiting = false;
      importLevel(imported);
    }
  }
}

void mouseDragged()
{
  if (mode == 3) {
    mover.drag();
  }
}

void mousePressed()
{
  if (mode == 2) {
    //automatic new mover
    //active = (active + 1) % movers.size();
    //mover = movers.get(active);
    
    mode = 1;
    
    int num = mover.positions.size();
    mover.reset();
    if (num == numFrames) {
      numFrames = getLongestMover().positions.size();
    }
    
    frame = 0;
    
  } else if (mode == 3) {
    ori = new PVector(mouseX, mouseY);
  }
}

void mouseReleased()
{
  if (mode == 1) {
    if (mover.positions.size() > 1) {
      mode = 2;
    } else { 
      mover.positions.clear();
    }
  }
  
  if (mode == 3) {
    ori = null;
  }
}

void keyPressed()
{
  if (key == CODED) {
    if (keyCode == UP) {
      int index = constrain(frame, 0, mover.rotations.size()-1);
      Float newVal = new Float(mover.rotations.get(index) + TWO_PI/60);
      mover.rotations.set(index, newVal);
    } else if (keyCode == DOWN) {
      int index = constrain(frame, 0, mover.rotations.size()-1);
      Float newVal = new Float(mover.rotations.get(index) - TWO_PI/60);
      mover.rotations.set(index, newVal);
    } else if (keyCode == LEFT) {
      if (frame > 0) {
        frame--;
      } else {
        frame = numFrames; 
      }
    } else if (keyCode == RIGHT) {
      if (frame < numFrames) {
        frame++;
      } else {
        frame = 0; 
      }
    }
  }
  
  if (key == 'c') {
    mover.type = (mover.type + 1) % actors.size();
  }
  
  if (key == 'f') {
    actionFrame = frame;
  }
  
  if (key == 'h') {
    help = !help;
  }
  
  if (key == 'i') {
    if (mode == 2) {
      importScene();
    }
  }
  
  if (key == 'd') {
    mover.state = !mover.state;
  }
  
  if (key == 'r') {
    if (mode == 2) {
      reset = true;
    }
  }
  
  if (keyCode >= 49 && keyCode <= 57) {
    int index = key-49;
    mover = movers.get(index);
    active = index;
  }
  
  if (key == ' ') {
    toggle = !toggle;
    if (mode == 2) {
      mode = 3;
    } else if (mode == 3) {
      mode = 2;
    }
  }
  
  if (key == 's') {
    DateFormat df = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
    Date d = new Date();
    date = df.format(d);
    saveLevel();
  }
}





