package ro.tibor.processing.gameJam;

import java.util.ArrayList;
import java.util.LinkedHashSet;
import java.util.List;

import com.jogamp.opengl.util.texture.spi.TGAImage;

import processing.core.PApplet;
import processing.core.PConstants;
import processing.core.PImage;
import processing.core.PVector;
import processing.data.*;

public class Level {
	PApplet p;
	JSONArray values;
	
	int frameLength;
	
	public int levelNumber;
	
	private ArrayList<PImage> triangleSprite;
	private ArrayList<PImage> circleSprite;
	private ArrayList<PImage> squareSprite;
	
	private PImage currentLevelImage;
	
	public ArrayList<Mover> listMovers;
	
	public List<LevelStateHandler_I> levelStateHandlers;
	private LevelStateHandler_I currentLevelStateHandler;
	
	public boolean isWon;
	
	public int futureFrame;
	
	public void setCurrentLevelStateHandler(
			LevelStateHandler_I currentLevelStateHandler) {
		this.currentLevelStateHandler = currentLevelStateHandler;
		this.currentLevelStateHandler.start();
	}

	Level(PApplet p, String levelName) {
		this.p = p;
		
		createSpriteSheets(p);
		createMoversFormJSON(p, levelName);
		generateLevelStateHandlers(p);		
	}

	private void createMoversFormJSON(PApplet p, String levelName) 
	{
		listMovers = new ArrayList<Mover>();
		
		JSONObject level = p.loadJSONObject(levelName);
		this.frameLength = level.getInt("length");
		
		int fritzIndex = level.getInt("fritz");
		int fritzTriggerFrame = level.getInt("frame");
		int futureFrame = level.getInt("future");
		this.futureFrame = futureFrame;
		
		JSONArray jsonMovers = level.getJSONArray("movers");
		
		for (int i=0; i < jsonMovers.size(); i++)
		{
			JSONObject jsonMover = jsonMovers.getJSONObject(i);
			int type = jsonMover.getInt("type");
			ArrayList<PImage> spriteSheet = spriteSheetFromIndex(type);
			
			JSONArray jsonFrames = jsonMover.getJSONArray("frames");
			boolean shouldDieAtEnd = jsonMover.getBoolean("state");
			
			ArrayList<PVector> listPosVectors = new ArrayList<>();
			ArrayList<Float> listRotations = new ArrayList<>();
			
			for (int j = 0; j < jsonFrames.size(); j++)
			{
				JSONObject jsonFrame = jsonFrames.getJSONObject(j);
				float x = jsonFrame.getFloat("x");
				float y = jsonFrame.getFloat("y");
				
				float rotation = jsonFrame.getFloat("rotation");

				listPosVectors.add(new PVector(x * p.width, y * p.height));
				listRotations.add(new Float(rotation));
			}
			
			//TODO triggerFrame
			//TODO add contidion with index
			
			boolean isFritz = i == fritzIndex;
			int triggerFrame = isFritz ? fritzTriggerFrame : 0;
			
			if (isFritz)
			{
				System.out.println("We are fritz");
				System.out.println("Trigger frame " + triggerFrame);
			}
			
			Mover mover = new Mover(p, listMovers, spriteSheet, listPosVectors.get(0),
					listRotations.get(0).floatValue(), shouldDieAtEnd, isFritz, triggerFrame);
			mover.listPositions = listPosVectors;
			mover.listRotations = listRotations;
			listMovers.add(mover);
		}
	}

	private ArrayList<PImage> spriteSheetFromIndex(int index)
	{
		ArrayList<PImage> result = null;
		switch (index) {
		case 0:
			result = triangleSprite;
			break;
		case 1:
			result = squareSprite;
			break;
		case 2:
			result = circleSprite;
			break;
		default:
			result = triangleSprite;
			break;
		}
		return result;
	}
	
	private void createSpriteSheets(PApplet p) 
	{
		triangleSprite = new ArrayList<>();
		for (int i = 0; i < 10; i++) {
			triangleSprite.add(p.loadImage("triangle/T" + i + ".png"));
		}
		squareSprite = new ArrayList<>();
		for (int i = 0; i < 9; i++) {
			squareSprite.add(p.loadImage("square/Square" + i + ".png"));
		}
		circleSprite = new ArrayList<>();
		for (int i = 0; i < 8; i++) {
			circleSprite.add(p.loadImage("circle/C" + i + ".png"));
		}
	}

	private void generateLevelStateHandlers(PApplet p) {
		levelStateHandlers = new ArrayList<>();
		LevelStateHandlerPast levelStateHandlerPast = new LevelStateHandlerPast(p, this);
		LevelStateHandlerFuture levelStateHandlerFuture = new LevelStateHandlerFuture(p, this);
		LevelStateHandlerNow levelStateHandlerNow = new LevelStateHandlerNow(p, this, frameLength);
		LevelStateHandlerOver handlerOver = new LevelStateHandlerOver(p, this);
		
		levelStateHandlers.add(levelStateHandlerPast);
		levelStateHandlers.add(levelStateHandlerFuture);
		levelStateHandlers.add(levelStateHandlerNow);
		levelStateHandlers.add(handlerOver);
		
		this.setCurrentLevelStateHandler(levelStateHandlers.get(0));
	}
	
	void update() 
	{
		/*
		if (p.frameCount % 30 == 0 && levelState != LevelState.NOW) {
			int systemVal = levelState.ordinal() + 1;
			LevelState enumVal = LevelState.values()[systemVal];
			levelState = enumVal;

			if (levelState == LevelState.NOW) {
				for (Mover m : listMovers) {
					m.init();
				}
			}
		}
		*/
		currentLevelStateHandler.update();
	}
	
	void draw()
	{
		currentLevelStateHandler.draw();
		if (currentLevelImage != null)
			p.image(currentLevelImage, p.width/2, p.height/2, p.width, p.height);
	}


	void click() 
	{
		this.currentLevelStateHandler.click();
	}
	
	/**************
	 * Callbacks
	 **************/
	
	void setCurrentLevelIndex(int currentLevelIndex)
	{
		currentLevelImage = p.loadImage("lvlnum/fg ex "+ currentLevelIndex +".png");
	}
	
	public void stateBegan(LevelStateHandler_I levelHandler)
	{
		
	}
	
	public void stateEnded(LevelStateHandler_I levelHandler)
	{
		int currentIndex = this.levelStateHandlers.indexOf(levelHandler);
		int nextStateIndex = (currentIndex + 1) % (levelStateHandlers.size()) ;
		
		this.setCurrentLevelStateHandler(levelStateHandlers.get(nextStateIndex));
	}
	
}
