package ro.tibor.processing.gameJam;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.Timer;

import processing.core.PApplet;
import processing.core.PImage;

public abstract class LevelStateHandler implements LevelStateHandler_I 
{
	PApplet p;
	Level level;
	int timeInMilis;
	
	int spriteSheetIndex;
	int foregroundIndex;
	
	ArrayList<PImage> backgroundImages;
	ArrayList<PImage> foregreoundImages;
	
	Timer timer;
	
	public LevelStateHandler(PApplet p, Level level) {
		this(p, level, 3000);
	}

	public LevelStateHandler(PApplet p, Level level, int durationInMilis) 
	{
		super();
		this.p = p;
		this.level = level;
		this.timeInMilis = durationInMilis;	
		this.setupSpriteSheets();
	}
	
	public void setupSpriteSheets()
	{
		this.backgroundImages = new ArrayList<>();
		spriteSheetIndex = 0;
		for (int i = 0; i < 6; i++) 
		{
			backgroundImages.add(p.loadImage("background/bg" + (i+1) + ".png"));
		}
		
		this.foregreoundImages = new ArrayList<>();
		for (int i = 0; i < 6; i++) 
		{
			foregreoundImages.add(p.loadImage("foreground/Fg" + i + ".png"));
		}
	}
	
	public void start()
	{
		
	}
	
	public void click()
	{ 
		onEnd(this);
	}
	
	@Override
	public void onBegin(LevelStateHandler_I sender) 
	{
		level.stateBegan(sender);
	}

	@Override
	public void onEnd(LevelStateHandler_I sender) 
	{
		if (timer != null) timer.stop();
		level.stateEnded(sender);
	}
	
	public void draw()
	{
		if (p.frameCount % 12 == 0)
		{
			spriteSheetIndex++;
			foregroundIndex = (int)p.random(6); 
		}
		
		p.tint(255);
		p.image(backgroundImages.get(spriteSheetIndex % 6 ), p.width * 0.5f, p.height * 0.5f, p.width , p.height);
		p.tint(255, 125);  // Display at half opacity
		p.image(foregreoundImages.get(foregroundIndex), p.width * 0.5f, p.height * 0.5f, p.width , p.height);
		p.tint(255);
	}	
}
